/**********************************************************************/
/*  Copyright (c) 2005-2014 by SAS Institute, Inc., Cary, NC, USA.    */
/*  All rights reserved.                                              */
/**********************************************************************/
/*  NAME:        orm_apply_dim_path.sas                               */
/*  VERSION:     6.1                                                  */
/*  DESCRIPTION: Apply dimension paths to the input table.            */
/*  PARAMETERS:  indsn=target table                                   */
/*               feature=target feature                               */
/*               prefix=prefix used in the dimension names when mult. */
/*                     dimension locations are needed in the output   */
/*                     (ex. incidents may want the discovery and event*/
/*                          location in the output table)             */
/*               prefix_label=Displayable name for the prefix parm    */
/*                            (ex. incidents would be "Discovery" for */
/*                                 "Discovery" location)              */
/*               keep_dimension=dimension to keep in the table.       */
/*                                   (used by CAUSE and CONTROL to    */
/*                                    keep only those dimensions for  */
/*                                    those objects)                  */
/*  PRODUCT:     SAS Enterprise GRC                                   */
/**********************************************************************/
%macro orm_apply_dim_path(indsn=, 
                          feature=,
                          prefix=,
                          prefix_label=,
                          keep_dimension=);

%put NOTE: {START: &sysmacroname};

  options fmtsearch=(opfmts opbifmts work);
     
  /* Create opcntl.cntl_dimensions and opcntl.cntl_features tables if */
  /* they do not exist */
  %if %sysfunc(exist(opcntl.cntl_dimensions)) %then 
  %do;
    %local nobs rc;
    %let dsid = %sysfunc(open(opcntl.cntl_dimensions));
    %if &dsid %then 
    %do;
      %let nobs  = %sysfunc(attrn(&dsid, nlobs));
      %let rc = %sysfunc(close(&dsid));
    %end;
    %else %let nobs  = 0;
 
    %if &nobs = 0 %then 
    %do;
      data opcntl.cntl_dimensions;
        infile OPCNTLD(dimensions.txt) dlm=',' firstobs=2;
        length dimension_nm $32;
        length short_prefix $8;
        input dimension_nm short_prefix levels;
      run;
    %end;
    %end;
    %else
    %do;
      data opcntl.cntl_dimensions;
        infile OPCNTLD(dimensions.txt) dlm=',' firstobs=2;
        length dimension_nm $32;
        length short_prefix $8;
        input dimension_nm short_prefix levels;
      run;
    %end;
     
    %if %sysfunc(exist(opcntl.cntl_features)) %then 
    %do;
      %local nobs rc;
      %let dsid = %sysfunc(open(opcntl.cntl_features));
      %if &dsid %then 
      %do;
        %let nobs  = %sysfunc(attrn(&dsid, nlobs));
        %let rc = %sysfunc(close(&dsid));
      %end;
      %else %let nobs  = 0;
 
      %if &nobs = 0 %then 
      %do;
        data features;
          infile OPCNTLD(features.txt) dlm=',' firstobs=2;
          length feature_nm $32;
          length dimension_nm $32;
          input feature_nm dimension_nm;
        run;
         
        /* Make sure that the dimensions of the "SYSTEM" feature */
        /* are a superset of the union of the dimensions of all  */
        /* other features. */
        proc sql noprint;
          create table all_feature_dims as
          select distinct dimension_nm
          from features;
        quit;
         
        data all_feature_dims;
          length feature_nm $32;
          set all_feature_dims;
          feature_nm = "SYSTEM";
        run;
         
        data features;
          set features(where=(feature_nm ne "SYSTEM"))
              all_feature_dims;
        run;
         
        /* Create a feature for each dimension. */
        data dim_features;
          length feature_nm $32;
          set opcntl.cntl_dimensions(keep=dimension_nm);
          feature_nm = dimension_nm;
        run;
         
        data features;
          set features dim_features;
        run;
         
        /* Pick up the number of levels from the dimensions table. */
        proc sort data=opcntl.cntl_dimensions;
          by dimension_nm;
        run;
         
        proc sort data=features nodupkey; /* in case we created duplicates */
          by dimension_nm feature_nm;
        run;
         
        data opcntl.cntl_features;
          merge features(in=left) opcntl.cntl_dimensions;
          by dimension_nm;
          if left;
        run;
         
        proc datasets nolist;
          delete features dim_features all_feature_dims;
        run;
        quit;
      %end;
    %end;
    %else
    %do;
      data features;
        infile OPCNTLD(features.txt) dlm=',' firstobs=2;
        length feature_nm $32;
        length dimension_nm $32;
        input feature_nm dimension_nm;
      run;
         
      /* Make sure that the dimensions of the "SYSTEM" feature */
      /* are a superset of the union of the dimensions of all */
      /* other features. */
      proc sql noprint;
        create table all_feature_dims as
        select distinct dimension_nm
        from features;
      quit;
         
      data all_feature_dims;
        length feature_nm $32;
        set all_feature_dims;
        feature_nm = "SYSTEM";
      run;
         
      data features;
        set features(where=(feature_nm ne "SYSTEM"))
        all_feature_dims;
      run;
         
      /* Create a feature for each dimension. */
       data dim_features;
         length feature_nm $32;
         set opcntl.cntl_dimensions(keep=dimension_nm);
         feature_nm = dimension_nm;
       run;
         
       data features;
         set features dim_features;
       run;
         
       /* Pick up the number of levels from the dimensions table. */
       proc sort data=opcntl.cntl_dimensions;
         by dimension_nm;
       run;
        
       proc sort data=features nodupkey; /* in case we created duplicates */
         by dimension_nm feature_nm;
       run;
         
       data opcntl.cntl_features;
         merge features(in=left) opcntl.cntl_dimensions;
         by dimension_nm;
         if left;
       run;
         
       proc datasets nolist;
         delete features dim_features all_feature_dims;
       run;
       quit;
    %end;
    
    /* Create the table w/ the MBCS multiplier if it does not exist */
    %if %sysfunc(exist(opcntl.cntl_char_multiplier)) %then 
    %do;
      %local nobs rc;
      %let dsid = %sysfunc(open(opcntl.cntl_char_multiplier));
      %if &dsid %then 
      %do;
        %let nobs  = %sysfunc(attrn(&dsid, nlobs));
        %let rc = %sysfunc(close(&dsid));
      %end;
      %else %let nobs  = 0;
    
      %if &nobs = 0 %then 
      %do;
        data opcntl.cntl_char_multiplier;
          infile OPCNTLD(char_multiplier.txt) firstobs=2;
          input char_multiplier;
        run;
      %end;
    %end;
    %else
    %do;
      data opcntl.cntl_char_multiplier;
        infile OPCNTLD(char_multiplier.txt) firstobs=2;
        input char_multiplier;
      run;
    %end;
    
    %global char_multiplier;
    
    /* get the MBCS multiplier */
    proc sql noprint;
      select CHAR_MULTIPLIER into : char_multiplier
      from opcntl.cntl_char_multiplier;
    quit;
    
    %let char_multiplier=&char_multiplier;
   
    /* Put the dimensions and levels into a list */
    proc sql noprint;     
      select count(*) into : DIMCOUNT
      from opcntl.cntl_features
      where upcase(feature_nm)=%upcase("&feature")
      
        %if &keep_dimension ne %str() %then
        %do;
          and upcase(dimension_nm) = "%upcase(&keep_dimension)"
        %end;
        ;

      %let DIMCOUNT=&DIMCOUNT;

      select dimension_nm into : DIMENSION1 - :DIMENSION&DIMCOUNT
      from opcntl.cntl_features
      where upcase(feature_nm)=%upcase("&feature")
      
        %if &keep_dimension ne %str() %then
        %do;
          and upcase(dimension_nm) = "%upcase(&keep_dimension)"
        %end;
        ;
    quit;

    /* Begin dimension format generation */
    %macro orm_create_dim_path_format(dim_nm=&&DIMENSION&I);
    
    %put NOTE: {START: &sysmacroname};
                
    %global &dim_nm._lvls;

      proc sql noprint;
        select levels into : &dim_nm._lvls
        from opcntl.cntl_features
        where upcase(feature_nm)=%upcase("&feature")
          and dimension_nm="&dim_nm";
      quit;

      %let &dim_nm._lvls=%trim(&&&dim_nm._lvls);

      %if %upcase(&dim_nm) = MANAGEMENT_ORG or %upcase(&dim_nm) = LEGAL_ORG %then         
        %let dim_column =organization_nm;        
      %else %let dim_column = &dim_nm._NM; 
         
      proc sql;
        create table &dim_nm._l as
        select 
          %if &prefix = %str() %then
          %do;
            &dim_nm._rk
          %end;
          %else
          %do;
            &dim_nm._rk as &prefix.&dim_nm._rk
          %end;
          %if %upcase(&dim_nm) = MANAGEMENT_ORG or %upcase(&dim_nm) = LEGAL_ORG %then
          %do;
            ,case
              when B.BUSINESS_OBJECT_NM="&dim_nm" and 
                A.&dim_nm._rk=B.business_object_rk and
                upcase(B.language_cd) = "&localecd" then B.name_txt
              else A.organization_nm
              end as organization_nm len=%eval(4000 * &char_multiplier)
          %end;
          %else
          %do;
            ,case
            when B.BUSINESS_OBJECT_NM="&dim_nm" and 
              A.&dim_nm._rk=B.business_object_rk and
              upcase(B.language_cd) = "&localecd" then B.name_txt
            else A.&dim_nm._NM
            end as &dim_nm._NM len=%eval(4000 * &char_multiplier)
          %end;
        from &src_lib..&dim_nm._l A left join
             &src_lib..full_sbo_trans B
        on B.BUSINESS_OBJECT_NM="&dim_nm" and 
           A.&dim_nm._rk=B.business_object_rk and
           upcase(B.language_cd)="&localecd";
      quit;

      data &dim_nm.fmtin(keep=fmtname type start label);
        length fmtname $32 type $1 start $64;
        retain fmtname "&prefix.&dim_nm.fmt" type 'N';
        set &dim_nm._l;
        
        %if &prefix = %str() %then
        %do;
          start = &dim_nm._rk;
        %end;
        %else
        %do;
          start = &prefix.&dim_nm._rk;
        %end;
        %if %upcase(&dim_nm) = MANAGEMENT_ORG or %upcase(&dim_nm) = LEGAL_ORG %then
        %do;
          label = trim(left(organization_nm));
        %end;
        %else
        %do;
          label = trim(left(&dim_nm._NM));
        %end;
      run;

      proc format cntlin=&dim_nm.fmtin library=opfmts;
      quit;
      run; 

      proc format cntlin=&dim_nm.fmtin library=opbifmts;
      quit;
      run; 
      
      proc format cntlin=&dim_nm.fmtin library=work;
      quit;
      run; 
      
      proc datasets nolist;
        delete &dim_nm._l &dim_nm.fmtin;
      quit;
      run;
  
    %put NOTE: {END: &sysmacroname};
  %mend orm_create_dim_path_format;
  
  /* Do not create dimension formats when loading LASR tables */
  %if not %symexist(load_lasr) %then
  %do;  
    /* Format all dimensions */        
    %do I=1 %to &DIMCOUNT.;  
      %orm_create_dim_path_format(dim_nm=&&DIMENSION&I);
    %end;  
    
    /* Apply formated dimension paths to the dimensions */ 
    data &indsn;
      set &indsn;
      %do J=1 %to &DIMCOUNT; 
        length &prefix.&&DIMENSION&J.._path $%eval(4000 * &char_multiplier); 
        format &prefix.&&DIMENSION&J.._rk &prefix.&&DIMENSION&J.._rk_1 - &prefix.&&DIMENSION&J.._rk_&&&&&&DIMENSION&J.._lvls;
         
        %do k=1 %to &&&&&&DIMENSION&J.._lvls;       
          %if &k=1 %then
          %do;
            if &prefix.&&DIMENSION&J.._rk_1 ne . then 
            do;
              &prefix.&&DIMENSION&J.._Path=trim(left(put(&prefix.&&DIMENSION&J.._rk_1, &prefix.&&DIMENSION&J..fmt.)));
               
              %if &prefix ne  %then
              %do;
                label &prefix.&&DIMENSION&J.._Path="&prefix_label &&DIMENSION&J %sysfunc(sasmsg(sashelp.ormonrpt,Path, NOQUOTE))";
              %end;
              %else
              %do;
                label &&DIMENSION&J.._Path="&&DIMENSION&J %sysfunc(sasmsg(sashelp.ormonrpt,Path, NOQUOTE))";
              %end;
            end;
          %end;
          %else
          %do;
            if &prefix.&&DIMENSION&J.._rk_&k ne . then 
            do;
              &prefix.&&DIMENSION&J.._Path=trim(left(&prefix.&&DIMENSION&J.._Path)) || ' > ' ||  trim(left(put(&prefix.&&DIMENSION&J.._rk_&k, &prefix.&&DIMENSION&J..fmt.))); 
              
              %if &prefix ne  %then
              %do;
                label &prefix.&&DIMENSION&J.._Path="&prefix_label &&DIMENSION&J %sysfunc(sasmsg(sashelp.ormonrpt,Path, NOQUOTE))";
              %end;
              %else
              %do;
                label &prefix.&&DIMENSION&J.._Path="&&DIMENSION&J %sysfunc(sasmsg(sashelp.ormonrpt,Path, NOQUOTE))";
              %end;
            end;
          %end;
        %end; 
      %end;   
       
      %if &prefix = %str() %then
      %do;
        %do r=1 %to &DIMCOUNT;   
          format &&DIMENSION&r.._rk  &&DIMENSION&r..fmt.; 
          %do s=1 %to &&&&&&DIMENSION&r.._lvls;              
            format &&&DIMENSION&r.._rk_&s &&DIMENSION&r..fmt.;
            label &&DIMENSION&r.._rk="&&DIMENSION&r %sysfunc(sasmsg(sashelp.ormonrpt,dim_level_nm, NOQUOTE))"
                  &&&DIMENSION&r.._rk_&s="&&&DIMENSION&r %sysfunc(sasmsg(sashelp.ormonrpt,dim_level_nm, NOQUOTE)) &s";
          %end;
        %end;
      %end;
      %else
      %do;
        %do r=1 %to &DIMCOUNT;   
          drop &prefix.&&DIMENSION&r.._rk; 
          %do s=1 %to &&&&&&DIMENSION&r.._lvls;              
            drop &prefix.&&&DIMENSION&r.._rk_&s;
          %end;
        %end;
      %end;
    run;
  %end;
  %else 
  %do;
    %if %upcase(&load_lasr) = TRUE %then
    %do;
      /* Format all dimensions */        
      %do I=1 %to &DIMCOUNT.;  
        %orm_create_dim_path_format(dim_nm=&&DIMENSION&I);
      %end;  
      
      /* Apply formated dimension paths to the dimensions */ 
      data &indsn;
        set &indsn;
        %do J=1 %to &DIMCOUNT; 
          length &prefix.&&DIMENSION&J.._path $%eval(4000 * &char_multiplier); 
          format &prefix.&&DIMENSION&J.._rk &prefix.&&DIMENSION&J.._rk_1 - &prefix.&&DIMENSION&J.._rk_&&&&&&DIMENSION&J.._lvls;
         
          %do k=1 %to &&&&&&DIMENSION&J.._lvls;       
            %if &k=1 %then
            %do;
              if &prefix.&&DIMENSION&J.._rk_1 ne . then 
              do;
                &prefix.&&DIMENSION&J.._Path=trim(left(put(&prefix.&&DIMENSION&J.._rk_1, &prefix.&&DIMENSION&J..fmt.)));
               
                %if &prefix ne  %then
                %do;
                  label &prefix.&&DIMENSION&J.._Path="&prefix_label &&DIMENSION&J %sysfunc(sasmsg(sashelp.ormonrpt,Path, NOQUOTE))";
                %end;
                %else
                %do;
                  label &&DIMENSION&J.._Path="&&DIMENSION&J %sysfunc(sasmsg(sashelp.ormonrpt,Path, NOQUOTE))";
                %end;
              end;
            %end;
            %else
            %do;
              if &prefix.&&DIMENSION&J.._rk_&k ne . then 
              do;
                &prefix.&&DIMENSION&J.._Path=trim(left(&prefix.&&DIMENSION&J.._Path)) || ' > ' ||  trim(left(put(&prefix.&&DIMENSION&J.._rk_&k, &prefix.&&DIMENSION&J..fmt.))); 
              
                %if &prefix ne  %then
                %do;
                  label &prefix.&&DIMENSION&J.._Path="&prefix_label &&DIMENSION&J %sysfunc(sasmsg(sashelp.ormonrpt,Path, NOQUOTE))";
                %end;
                %else
                %do;
                  label &prefix.&&DIMENSION&J.._Path="&&DIMENSION&J %sysfunc(sasmsg(sashelp.ormonrpt,Path, NOQUOTE))";
                %end;
              end;
            %end;
          %end; 
        %end;   
       
        %if &prefix = %str() %then
        %do;
          %do r=1 %to &DIMCOUNT; 
            length un_&&DIMENSION&r.._rk $%eval(4000 * &char_multiplier);  
            un_&&DIMENSION&r.._rk=trim(left(put(&&DIMENSION&r.._rk, &&DIMENSION&r..fmt.))); 
            drop &&DIMENSION&r.._rk;
            label un_&&DIMENSION&r.._rk="&&DIMENSION&r %sysfunc(sasmsg(sashelp.ormonrpt,dim_level_nm, NOQUOTE))";
            rename un_&&DIMENSION&r.._rk=&&DIMENSION&r.._rk;

            %do s=1 %to &&&&&&DIMENSION&r.._lvls;    
              length un_&&&DIMENSION&r.._rk_&s $%eval(4000 * &char_multiplier);      
              un_&&&DIMENSION&r.._rk_&s=trim(left(put(&&&DIMENSION&r.._rk_&s, &&DIMENSION&r..fmt.))); 
              drop &&&DIMENSION&r.._rk_&s;
              label un_&&&DIMENSION&r.._rk_&s="&&&DIMENSION&r %sysfunc(sasmsg(sashelp.ormonrpt,dim_level_nm, NOQUOTE)) &s";
              rename un_&&&DIMENSION&r.._rk_&s=&&&DIMENSION&r.._rk_&s;                
            %end;
          %end;
        %end;
        %else
        %do;
          %do r=1 %to &DIMCOUNT;   
            drop &prefix.&&DIMENSION&r.._rk; 
            %do s=1 %to &&&&&&DIMENSION&r.._lvls;              
              drop &prefix.&&&DIMENSION&r.._rk_&s;
            %end;
          %end;
        %end;
      run;
    %end;
  %end;  
%put NOTE: {END: &sysmacroname}; 

%mend orm_apply_dim_path; 
